Attribute VB_Name = "Tokens"
Option Explicit

' Clicks this close to an object hit the object.
Public Const CLOSE_DIST = 30

' List of errors.
Private m_Errors As String
' Draw an ellipse in the box.
Public Sub DrawEllipse(ByVal Canvas As Object, ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single)
Dim Cx As Single
Dim Cy As Single
Dim wid As Single
Dim hgt As Single
Dim r As Single

    wid = Abs(X1 - X2)
    hgt = Abs(Y1 - Y2)
    If wid < 1 Or hgt < 1 Then Exit Sub

    Cx = (X1 + X2) / 2
    Cy = (Y1 + Y2) / 2
    If wid > hgt Then
        r = wid / 2
    Else
        r = hgt / 2
    End If

    Canvas.Circle (Cx, Cy), r, , , , hgt / wid
End Sub
' Display and clear any errors.
Public Sub ErrorsDisplay(ByVal err_source As String)
    If Len(m_Errors) > 0 Then
        Err.Raise vbObjectError + 1001, _
            err_source, m_Errors
        m_Errors = ""
    End If
End Sub
' Remove a token from txt. Tokens are stored in
' the form "token_name(token_value)".
Public Sub GetToken(ByRef txt As String, ByRef token_name As String, ByRef token_value As String)
Dim open_pos As Integer
Dim close_pos As Integer
Dim num_open As Integer

    ' Find the open parenthesis and the token name.
    open_pos = InStr(txt, "(")

    ' Find the corresponding close parenthesis.
    num_open = 1
    For close_pos = open_pos + 1 To Len(txt)
        Select Case Mid$(txt, close_pos, 1)
            Case "("
                num_open = num_open + 1
            Case ")"
                num_open = num_open - 1
                If num_open = 0 Then Exit For
        End Select
    Next close_pos

    ' See if we found the closing parenthesis.
    If num_open = 0 Then
        ' We found it. Save the token name and value.
        token_name = Trim$(Left$(txt, open_pos - 1))
        token_value = Trim$(Mid$(txt, open_pos + 1, close_pos - open_pos - 1))
        txt = Trim$(Mid$(txt, close_pos + 1))
    Else
        ' We did not find it.
        ' Save an error.
        ErrorRecord "No matching ')' in:" & _
            vbCrLf & "    " & Mid$(txt, close_pos)

        ' Blank everything.
        token_name = ""
        token_value = ""
        txt = ""
    End If
End Sub

' Save this error.
Public Sub ErrorRecord(ByVal txt As String)
    If Len(m_Errors) > 0 Then m_Errors = m_Errors & vbCrLf
    m_Errors = m_Errors & txt
End Sub

' Calculate the distance from a point to a segment.
Public Function DistToSegment(ByVal px As Single, ByVal py As Single, ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single) As Single
Dim dx As Single
Dim dy As Single
Dim t As Single

    dx = X2 - X1
    dy = Y2 - Y1
    If dx = 0 And dy = 0 Then
        ' It's a point not a line segment.
        dx = px - X1
        dy = py - Y1
        DistToSegment = Sqr(dx * dx + dy * dy)
        Exit Function
    End If

    t = (dx * (px - X1) + dy * (py - Y1)) / (dx * dx + dy * dy)
    If t < 0 Then
        dx = px - X1
        dy = py - Y1
    ElseIf t > 1 Then
        dx = px - X2
        dy = py - Y2
    Else
        X2 = X1 + t * dx
        Y2 = Y1 + t * dy
        dx = px - X2
        dy = py - Y2
    End If
    DistToSegment = Sqr(dx * dx + dy * dy)
End Function


