VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ld_Picture"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' All of the drawing objects.
Private m_DrawingObjects As Collection

' The currently selected object.
Private m_SelectedObject As Object

Private m_FileName As String
Private m_FileTitle As String
Private m_Dirty As Boolean

' The object on which we draw.
Private m_Canvas As Object
' Load the data file.
Public Sub DataLoad(ByVal file_name As String, ByVal file_title As String)
Dim stage As String
Dim new_objects As Collection
Dim txt As String
Dim fnum As Integer
Dim token_name As String
Dim token_value As String
Dim draw_object As Object

    On Error GoTo LoadError
    stage = "opening file '" & file_name & "'"
    fnum = FreeFile
    Open file_name For Input As #fnum

    ' Get the text.
    stage = "getting file text"
    txt = Input$(LOF(fnum), #fnum)
    Close #fnum

    ' Remove vbCrLf.
    txt = Replace(txt, vbCrLf, "")

    ' Process the text.
    Set new_objects = New Collection
    stage = "processing text"
    Do While Len(txt) > 0
        ' Get the next token.
        GetToken txt, token_name, token_value

        ' Process the token.
        Select Case token_name
            Case "Circle"
                Set draw_object = New ld_Circle
            Case "Line"
                Set draw_object = New ld_Line
            Case "Rectangle"
                Set draw_object = New ld_Rectangle
            Case "Ellipse"
                Set draw_object = New ld_Ellipse
            Case Else
                ErrorRecord "Unknown object type '" & token_name & "'"
                Set draw_object = Nothing
        End Select

        ' Make the object initialize itself.
        If Not (draw_object Is Nothing) Then
            draw_object.Serialization = token_value
            Set draw_object.Canvas = m_Canvas
            new_objects.Add draw_object
        End If
    Loop

    ' Replace the current objects with the new ones.
    stage = "redrawing"
    Set m_DrawingObjects = new_objects

    ' Save and display the file name.
    m_FileName = file_name
    m_FileTitle = file_title

    Exit Sub

LoadError:
    ErrorRecord "Error " & stage & vbCrLf & Err.Description
    ErrorsDisplay "ld_Picture.DataLoad"
End Sub
' Save the data using the current file name.
Public Sub DataSave(ByVal file_name As String, ByVal file_title As String)
Dim fnum As Integer
Dim stage As String
Dim draw_object As Object

    On Error GoTo SaveError
    stage = "opening file '" & file_name & "'"
    fnum = FreeFile
    Open file_name For Output As #fnum

    ' Save each object's serialization.
    stage = "saving serializations"
    For Each draw_object In m_DrawingObjects
        Print #fnum, draw_object.Serialization
    Next draw_object

    Close #fnum
    m_FileName = file_name
    m_FileTitle = file_title

    m_Dirty = False
    Exit Sub

SaveError:
    ErrorRecord "Error " & stage & vbCrLf & Err.Description
    ErrorsDisplay "ld_Picture.DataSave"
End Sub
' Return the loaded file's name.
Public Property Get FileName() As String
    FileName = m_FileName
End Property
' Return the loaded file's title.
Public Property Get FileTitle() As String
    FileTitle = m_FileTitle
End Property
' Initialize the Picture object.
Public Sub Initialize(ByVal Canvas As Object)
    Set m_Canvas = Canvas
End Sub

' Return True if the dats is dirty.
Public Function IsDirty() As Boolean
    IsDirty = m_Dirty
End Function

' Make a new circle.
Public Function NewCircle(ByVal Cx As Single, ByVal Cy As Single, ByVal Radius As Single) As ld_Circle
Dim new_circle As ld_Circle

    Set new_circle = New ld_Circle
    With new_circle
        .X0 = Cx
        .Y0 = Cy
        .Radius = Radius
        Set .Canvas = m_Canvas
    End With

    ' Add the new object to the collection.
    m_DrawingObjects.Add new_circle

    SetDirty

    ' Return the new object.
    Set NewCircle = new_circle
End Function
' Make a new ellipse.
Public Function NewEllipse(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single) As ld_Ellipse
Dim new_Ellipse As ld_Ellipse

    Set new_Ellipse = New ld_Ellipse
    With new_Ellipse
        .SetCoordinates X1, Y1, X2, Y2
        Set .Canvas = m_Canvas
    End With

    ' Add the new object to the collection.
    m_DrawingObjects.Add new_Ellipse

    SetDirty

    ' Return the new object.
    Set NewEllipse = new_Ellipse
End Function
' Make a new rectangle.
Public Function NewRectangle(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single) As ld_Rectangle
Dim new_rectangle As ld_Rectangle

    Set new_rectangle = New ld_Rectangle
    With new_rectangle
        .SetCoordinates X1, Y1, X2, Y2
        Set .Canvas = m_Canvas
    End With

    ' Add the new object to the collection.
    m_DrawingObjects.Add new_rectangle

    SetDirty

    ' Return the new object.
    Set NewRectangle = new_rectangle
End Function

' Make a new line.
Public Function NewLine(ByVal X1 As Single, ByVal Y1 As Single, ByVal X2 As Single, ByVal Y2 As Single) As ld_Line
Dim new_line As ld_Line

    Set new_line = New ld_Line
    With new_line
        .X1 = X1
        .Y1 = Y1
        .X2 = X2
        .Y2 = Y2
        Set .Canvas = m_Canvas
    End With

    ' Add the new object to the collection.
    m_DrawingObjects.Add new_line

    SetDirty

    ' Return the new object.
    Set NewLine = new_line
End Function
' Find the object at this position.
Public Function ObjectAt(ByVal X As Single, ByVal Y As Single) As Object
Dim i As Integer

    ' See if an object is here. Look from top
    ' to bottom.
    For i = m_DrawingObjects.Count To 1 Step -1
        If m_DrawingObjects(i).IsAt(X, Y) Then Exit For
    Next i

    If i > 0 Then
        Set ObjectAt = m_DrawingObjects(i)
    Else
        Set ObjectAt = Nothing
    End If
End Function
' Set the selected object.
Public Property Set SelectedObject(ByVal obj As Object)
    ' Deselect the previously selected object if any.
    If Not m_SelectedObject Is Nothing Then
        m_SelectedObject.Selected = False
    End If

    ' Select the new object.
    Set m_SelectedObject = obj
    If Not m_SelectedObject Is Nothing Then
        m_SelectedObject.Selected = True
    End If
End Property
' Return a reference to the selected object.
Public Property Get SelectedObject() As Object
    Set SelectedObject = m_SelectedObject
End Property

' Move the selected object to the bottom
' of the drawing order.
Public Sub SelectedObjectToBottom()
Dim selected_object As Object

    ' Remove the object.
    Set selected_object = m_SelectedObject
    RemoveSelectedObject

    ' Readd the object at the bottom of the list.
    If m_DrawingObjects.Count > 0 Then
        m_DrawingObjects.Add selected_object, , 1
    Else
        m_DrawingObjects.Add selected_object
    End If

    ' Reselect the object.
    Set SelectedObject = selected_object
End Sub
' Move the selected object to the top
' of the drawing order.
Public Sub SelectedObjectToTop()
Dim selected_object As Object

    ' Remove the object.
    Set selected_object = m_SelectedObject
    RemoveSelectedObject

    ' Readd the object at the top of the list.
    m_DrawingObjects.Add selected_object

    ' Reselect the object.
    Set SelectedObject = selected_object
End Sub
' Flag the data as dirty.
Public Sub SetDirty()
    m_Dirty = True
End Sub
' Redraw all objects.
Public Sub Redraw()
Dim obj As Object

    m_Canvas.Cls

    For Each obj In m_DrawingObjects
        obj.Draw
    Next obj
End Sub

' Remove the selected object from the
' m_DrawingObjects collection.
Public Sub RemoveSelectedObject()
Dim i As Integer

    ' Find the selected object's current position.
    For i = 1 To m_DrawingObjects.Count
        If m_DrawingObjects(i) Is m_SelectedObject Then Exit For
    Next i

    ' Remove the object.
    m_DrawingObjects.Remove i

    Set SelectedObject = Nothing
End Sub

Private Sub Class_Initialize()
    Set m_DrawingObjects = New Collection
    Set m_SelectedObject = Nothing
    m_FileName = ""
    m_FileTitle = ""
    m_Dirty = False
End Sub


